﻿namespace Phoenix.Main.Frame
{
    using System.Windows.Forms;

    using Phoenix.Log.Writer;
    using Phoenix.Main.Properties;
    using Phoenix.Translation;
    using Phoenix.View;

    /// <summary>
    /// Provide management tools for configuration of Phoenix.
    /// </summary>
    public partial class ConfigurationBox : TranslatedForm
    {
        #region Fields

        private const string NodeAppearance = "appearance";
        private const string NodeSearchBox = "searchbox";
        private const string NodeShortcuts = "shortcuts";

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="ConfigurationBox"/> class.
        /// </summary>
        public ConfigurationBox()
        {
            InitializeComponent();
            base.Logger = LogManager.GetLogger(typeof(ConfigurationBox));

            this.tv_Menu.Nodes["appearance"].Text = "Appearance".Translate();
            this.tv_Menu.Nodes["appearance"].Nodes["searchbox"].Text = "Search box".Translate();
            this.tv_Menu.Nodes["shortcuts"].Text = "Shortcuts".Translate();
            this.tv_Menu.ExpandAll();
            this.HideConfigurationHandlers();
        }

        #endregion Constructors

        #region Methods

        private void btn_Cancel_Click(object sender, System.EventArgs e)
        {
            this.searchBoxNode.Rollback();
            this.shortcutsNode.Rollback();
            this.appearanceNode.Rollback();
        }

        private void btn_OK_Click(object sender, System.EventArgs e)
        {
            this.appearanceNode.SaveConfiguration();
            this.searchBoxNode.SaveConfiguration();
            this.shortcutsNode.SaveConfiguration();

            base.DialogResult = DialogResult.OK;
            this.Close();
        }

        private void HideConfigurationHandlers()
        {
            foreach (Control control in sc_Splitter.Panel2.Controls[0].Controls)
            {
                if (control.Name == "pnl_panel")
                {
                    foreach (Control item in control.Controls)
                    {
                        item.Visible = false;
                        item.Dock = DockStyle.None;
                    }
                }
            }
        }

        private void tv_Menu_AfterSelect(object sender, TreeViewEventArgs e)
        {
            this.HideConfigurationHandlers();

            if (e.Node.Name == NodeSearchBox)
            {
                this.searchBoxNode.Visible = true;
                this.searchBoxNode.Dock = DockStyle.Fill;
            }
            else if (e.Node.Name == NodeShortcuts)
            {
                this.shortcutsNode.Visible = true;
                this.shortcutsNode.Dock = DockStyle.Fill;
            }
            else if (e.Node.Name == NodeAppearance)
            {
                this.appearanceNode.Visible = true;
                this.appearanceNode.Dock = DockStyle.Fill;
            }
        }

        #endregion Methods
    }
}