﻿namespace Phoenix.Model.Management
{
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;

    /// <summary>
    /// Manage drugs
    /// </summary>
    public class PathologyTypeModel : Manager<PathologyType>
    {
        #region Fields

        private IPathologyTypeInputPresenter presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PathologyTypeModel"/> class.
        /// </summary>
        /// <param name="view">The view.</param>
        public PathologyTypeModel(IPathologyTypeInputPresenter view)
            : base(RepositoryFactory.Get<IPathologyTypeRepository>())
        {
            this.presenter = view;
            this.presenter.Inserted += (sender, e) => this.Repository.Create(this.presenter.ItemToManage);
            this.presenter.Updated += (sender, e) => this.Repository.Update(this.presenter.ItemToManage);
            this.presenter.Deleted += (sender, e) => this.Delete(this.presenter.ItemToManage);
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns>
        /// 	<c>True</c> if the data have passed the validation;
        /// otherwise <c>False</c>
        /// </returns>
        public override bool ValidateData()
        {
            this.presenter.ClearError();
            bool isValid = true;
            if (string.IsNullOrEmpty(this.presenter.ItemToManage.Title))
            {
                this.presenter.SetError(PathologyTypeFailedOn.NoName, "The pathology type doesn't have a name".Translate());
                isValid = false;
            }
            return isValid;
        }

        /// <summary>
        /// Process to execute when a delete is raised.
        /// </summary>
        /// <param name="item">The item to delete.</param>
        protected override void Delete(PathologyType item)
        {
            if (this.Repository.IsReferenced(this.presenter.ItemToManage))
            {
                this.presenter.SetError(PathologyTypeFailedOn.Referenced, "Impossible to delete this item, it is referenced somewhere else.");
            }
            else
            {
                base.Delete(item);
            }
        }

        #endregion Methods
    }
}