﻿namespace Phoenix.Model.Management
{
    using Phoenix.Contracts;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal;
    using Phoenix.Presenter;
    using Phoenix.Presenter.Interfaces.Input;

    /// <summary>
    /// Provides features to manage the practice.
    /// </summary>
    public class PracticeModel : Manager<Practice>
    {
        #region Fields

        private IPracticeInputPresenter presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PracticeModel"/> class.
        /// </summary>
        /// <param name="presenter">The presenter.</param>
        public PracticeModel(IPracticeInputPresenter presenter)
            : base(RepositoryFactory.Get<IPracticeRepository>())
        {
            this.presenter = presenter;
            this.presenter.Inserted += (sender, e) => this.Insert(this.presenter.ItemToManage);
            this.presenter.Updated += (sender, e) => this.Update(this.presenter.ItemToManage);
            this.presenter.Deleted += (sender, e) => this.Delete(this.presenter.ItemToManage);
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns><c>True</c> if the data have passed the validation;
        /// otherwise <c>False</c></returns>
        public override bool ValidateData()
        {
            Assert.IsNotNull(this.presenter.ItemToManage, "ItemToManage", "The practice wasn't set before validation!");
            var result = true;

            this.presenter.ClearError();
            string text = this.presenter.ItemToManage.Title;
            if (string.IsNullOrEmpty(text))
            {
                string msg = "A practice can't have an empty name!";
                this.presenter.SetError(PracticeFailedOn.EmptyTitle, msg);
                result = false;
            }
            return result;
        }

        /// <summary>
        /// Process to execute when a delete is raised.
        /// </summary>
        /// <param name="item">The item to delete.</param>
        protected override void Delete(Practice item)
        {
            if (this.Repository.IsReferenced(this.presenter.ItemToManage))
            {
                this.presenter.SetError(PracticeFailedOn.Referenced, "Impossible to delete this item, it is referenced somewhere else.");
            }
            else
            {
                base.Delete(item);
            }
        }

        #endregion Methods
    }
}