﻿namespace Phoenix.Model.Core.Management
{
    using Phoenix.Contracts;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Dal;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces.Input;

    /// <summary>
    /// Manage the users
    /// </summary>
    public class UserModel
    {
        #region Fields

        private IUserInputPresenter presenter;
        private IUserRepository repository = RepositoryFactory.Get<IUserRepository>();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="UserModel"/> class.
        /// </summary>
        /// <param name="presenter">The presenter.</param>
        public UserModel(IUserInputPresenter presenter)
        {
            this.presenter = presenter;
            this.presenter.ActionOccured += (sender, e) => this.ProcessAction(e.Data);
        }

        #endregion Constructors

        #region Methods

        private void Delete()
        {
            this.repository.Delete(this.presenter.ItemToManage);
        }

        private void Insert()
        {
            this.repository.Create(this.presenter.ItemToManage);
        }

        private void ProcessAction(Phoenix.Presenter.Helpers.Usage usage)
        {
            switch (usage)
            {
                case Usage.Display: break;
                case Usage.Insert:
                    this.Insert();
                    break;
                case Usage.Update:
                    this.Update();
                    break;
                case Usage.Delete:
                    this.Delete();
                    break;
                default:
                    Assert.FailsOnEnumeration(usage);
                    break;
            }
        }

        private void Update()
        {
            this.repository.Update(this.presenter.ItemToManage);
        }

        #endregion Methods
    }
}