﻿namespace Phoenix.Model.Dal.Repositories
{
    using System;
    using System.Collections.Generic;
    using System.Linq;

    using NHibernate.Linq;

    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;

    public class PrescriptionRepository : Repository<Prescription>, IPrescriptionRepository
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PrescriptionRepository"/> class.
        /// </summary>
        public PrescriptionRepository()
            : base("Prescription", "CreationDate")
        {
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Determines whether the specified item is referenced.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns>
        /// 	<c>true</c> if the specified item is referenced; otherwise, <c>false</c>.
        /// </returns>
        public override bool IsReferenced(Prescription item)
        {
            return (from Patient p in this.Session.Linq<Patient>()
                    where p.Prescriptions.Contains(item)
                    select p).Count() > 0;
        }

        /// <summary>
        /// Searches the list of prescription for the specified patient
        /// from the specified 'from' date to the specified 'to' date.
        /// </summary>
        /// <param name="patient">The patient.</param>
        /// <param name="from">The 'from' date.</param>
        /// <param name="to">The "to" date.</param>
        /// <returns></returns>
        public List<Prescription> Search(Patient patient, DateTime start, DateTime end)
        {
            return (from Prescription p in patient.Prescriptions
                    where p.CreationDate >= start && p.CreationDate <= end
                    select p).ToList();
        }

        #endregion Methods
    }
}