﻿namespace Phoenix.Model.Dal.Repositories
{
    using System.Linq;

    using NHibernate.Linq;

    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;

    /// <summary>
    /// Repository for relations type
    /// </summary>
    public class RelationTypeRepository : Repository<RelationType>, IRelationTypeRepository
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="RelationRepository"/> class.
        /// </summary>
        public RelationTypeRepository()
            : base("RelationType", "Title")
        {
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Determines whether the specified item is referenced.
        /// </summary>
        /// <param name="item">The item.</param>
        /// <returns>
        /// 	<c>true</c> if the specified item is referenced; otherwise, <c>false</c>.
        /// </returns>
        public override bool IsReferenced(RelationType item)
        {
            return (from patient in this.Session.Linq<Patient>()
                    where patient.FamilyMembers.HasType(item)
                    select patient).Count() > 0;
        }

        #endregion Methods
    }

    static class Helpers
    {
        #region Methods

        public static bool HasType(this Relation[] relations, RelationType type)
        {
            return (from relation in relations
                    where relation.Type == type
                    select relation).Count() > 0;
        }

        #endregion Methods
    }
}