﻿namespace Phoenix.Presenter.Interfaces
{
    using Phoenix.Model.Bll.Domain;

    #region Enumerations

    public enum BmiFailedOn
    {
        InsertDate,
        TooSmallWeight,
        TooBigWeight,
        TooSmallHeight,
        TooBigHeight,
    }

    #endregion Enumerations

    /// <summary>
    /// Implements a control that displays Bmi entries.
    /// </summary>
    public interface IBmiDisplayPresenter
    {
        #region Properties

        /// <summary>
        /// Gets or sets the bmi to manage.
        /// </summary>
        /// <value>The bmi to manage.</value>
        BMI BmiToManage
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the height of tha patient to manage.
        /// </summary>
        /// <value>The height of the patient.</value>
        int LastHeight
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Clears previous error on validation failed.
        /// </summary>
        void ClearErrors();

        /// <summary>
        /// Sets the error explaining the validation fail for
        /// the specified control.
        /// </summary>
        /// <param name="failure">Indicate where the validation failed.</param>
        /// <param name="message">The message.</param>
        void Invalidate(BmiFailedOn failure, string message);

        #endregion Methods
    }
}