﻿namespace Phoenix.Presenter.Interfaces
{
    using System;
    using System.Collections.Generic;

    using Phoenix.Presenter.Sessions;

    /// <summary>
    /// Provides basic feature for a sessions disposer.
    /// </summary>
    public interface ISessionDisposer
    {
        #region Events

        /// <summary>
        /// Occurs when user is closing sessions.
        /// </summary>
        event EventHandler SessionsClosing;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets all the opened sessions.
        /// </summary>
        /// <value>The opened sessions.</value>
        SessionCollection OpenedSessions
        {
            get;
        }

        /// <summary>
        /// Gets the list of sessions to close.
        /// </summary>
        /// <value>The closed patients.</value>
        List<Session> SessionsToClose
        {
            get;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Indicates the user the application has finished working.
        /// Typically hide the sandglass cursor show the default cursor
        /// </summary>
        void EndGuiWait();

        /// <summary>
        /// Add a list of opened sessions.
        /// </summary>
        /// <param name="sessions">The sessions.</param>
        void SetOpenedSessions(SessionCollection sessions);

        /// <summary>
        /// Indicates the user the application is working.
        /// Typically show the sandglass cursor
        /// </summary>
        void StartGuiWait();

        #endregion Methods
    }
}