﻿namespace Phoenix.Presenter.Interfaces.Interfaces
{
    using System;
    using System.Collections.Generic;

    using Phoenix.Helpers.Misc;

    /// <summary>
    /// Represent the Presenter for a generic searchbox
    /// </summary>
    /// <typeparam name="T">The type the search box work on.</typeparam>
    public interface ISearchBox<T>
    {
        #region Events

        /// <summary>
        /// Occurs when the user fire a search in the GUI
        /// </summary>
        event EventHandler<EventArgs<T>> Searching;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the criterion for the search.
        /// </summary>
        /// <value>The criterion.</value>
        object Criterion
        {
            get;
        }

        /// <summary>
        /// Gets the item the user selected on double click or on click
        /// on the 'select' button.
        /// </summary>
        object SelectedItem
        {
            get;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Indicates the user the application has finished working.
        /// Typically hide the sandglass cursor show the default cursor
        /// </summary>
        void EndGuiWait();

        /// <summary>
        /// Fills the specified items into the GUI.
        /// </summary>
        /// <param name="items">The items to load into the SearchBox.</param>
        void Fill(IList<T> items);

        /// <summary>
        /// Indicates the user the application is working.
        /// Typically show the sandglass cursor
        /// </summary>
        void StartGuiWait();

        /// <summary>
        /// Warns the user when search return zero result.
        /// </summary>
        void WarnOnZeroResult();

        #endregion Methods
    }
}