﻿namespace Phoenix.Test.MedicalCardRelease
{
    using System;
    using System.Drawing;

    using NUnit.Framework;

    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Bll.Interface;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Interfaces.Input;

    using Rhino.Mocks;
    using Rhino.Mocks.Interfaces;

    [TestFixture]
    public class KeywordTestSuite
    {
        #region Constructors

        public KeywordTestSuite()
        {
            this.Mocks = new MockRepository();
        }

        #endregion Constructors

        #region Properties

        private int EventCounter
        {
            get;
            set;
        }

        private MockRepository Mocks
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        [Test]
        public void CanRaiseEvents()
        {
            this.EventCounter = 0;
            IKeywordInputPresenter view = this.Mocks.DynamicMock<IKeywordInputPresenter>();

            //This is the item to manage the user create from GUI
            KeywordGroup kwg = new KeywordGroup();
            kwg.Color = Color.Blue;
            kwg.Keywords.Add("One");
            kwg.Keywords.Add("two");
            kwg.Keywords.Add("three");

            // It simulates the link with the GUI and the property.
            // Whenever the presenter needs the ItemToManage, the GUI
            // binds it automotically
            view.ItemToManage = kwg;

            using (this.Mocks.Record())
            {
                view.Inserted += (sender, e) =>
                {
                    this.EventCounter++;
                };

                //Now user fires the event
                IEventRaiser raiseAdd = LastCall.IgnoreArguments().GetEventRaiser();
                raiseAdd.Raise(this, EventArgs.Empty);
            }

            Assert.AreEqual(1, this.EventCounter, string.Format(
                "Events ware fired '{0}' times but 1 times was expected!",
                EventCounter));
        }

        [Test]
        public void OnInvalidData()
        {
            #region Setup
            IKeywordInputPresenter view = this.Mocks.DynamicMock<IKeywordInputPresenter>();
            IRepository<KeywordGroup> repository = this.Mocks.DynamicMock<IRepository<KeywordGroup>>();
            KeywordModel presenter = new KeywordModel(view, repository);

            KeywordGroup invalid = new KeywordGroup()
            {
                RawKeywords = string.Empty,
                Title = string.Empty
            };
            #endregion

            using (this.Mocks.Record())
            {
                SetupResult.For(view.ItemToManage).Return(invalid);
            }

            bool result = presenter.ValidateData();
            Assert.AreEqual(result, false, "The validation should fail!");
        }

        [Test]
        public void OnValidData()
        {
            #region Setup
            IKeywordInputPresenter view = this.Mocks.DynamicMock<IKeywordInputPresenter>();
            IRepository<KeywordGroup> repository = this.Mocks.DynamicMock<IRepository<KeywordGroup>>();
            KeywordModel presenter = new KeywordModel(view, repository);

            KeywordGroup valid = new KeywordGroup()
            {
                RawKeywords = "aaa bbb ccc",
                Title = "title"
            };
            #endregion

            using (this.Mocks.Record())
            {
                SetupResult.For(view.ItemToManage).Return(valid);
            }

            bool result = presenter.ValidateData();
            Assert.AreEqual(result, true, "The validation should succeed!");
        }

        #endregion Methods
    }
}