﻿namespace Phoenix.View.Administration
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Windows.Forms;

    using Phoenix.Contracts;
    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Administration;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Presenter.Helpers;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Presenter.Interfaces.Input;

    /// <summary>
    /// Gui item to administrate all the informations of the database.
    /// </summary>
    public partial class AdministrationBox : TranslateableForm, IAdministrationPresenter
    {
        #region Fields

        private IList<Doctor> doctors = new List<Doctor>();
        private IList<DoctorType> doctorTypes = new List<DoctorType>();
        private IList<Drug> drugs = new List<Drug>();
        private IList<DrugType> drugTypes = new List<DrugType>();
        private IList<Insurance> insurances = new List<Insurance>();
        private IList<MedicalCardType> medicalCardTypes = new List<MedicalCardType>();
        private IList<Pathology> pathologies = new List<Pathology>();
        private IList<PathologyType> pathologyTypes = new List<PathologyType>();
        private IList<Practice> practices = new List<Practice>();
        private AdministrationModel presenter;
        private IList<RelationType> relationTypes = new List<RelationType>();
        private IList<Reputation> reputations = new List<Reputation>();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="AdministrationBox"/> class.
        /// </summary>
        public AdministrationBox()
        {
            InitializeComponent();
            this.presenter = new AdministrationModel(this);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when the GUI item is refreshed.
        /// </summary>
        public event EventHandler<RefreshEventArgs> Refreshed;

        #endregion Events

        #region Properties

        /// <summary>
        /// Gets or sets the doctors.
        /// </summary>
        /// <value>The doctors.</value>
        public IList<Doctor> Doctors
        {
            get
            {
                return this.doctors;
            }
            set
            {
                this.doctors = value;
                this.RefreshDoctor(value);
            }
        }

        /// <summary>
        /// Gets or sets the doctor types.
        /// </summary>
        /// <value>The doctor types.</value>
        public IList<DoctorType> DoctorTypes
        {
            get
            {
                return this.doctorTypes;
            }
            set
            {
                this.doctorTypes = value;
                this.RefreshDoctorType(value);
            }
        }

        /// <summary>
        /// Gets or sets the drugs.
        /// </summary>
        /// <value>The drugs.</value>
        public IList<Drug> Drugs
        {
            get
            {
                return this.drugs;
            }
            set
            {
                this.drugs = value;
                this.RefreshDrug(value);
            }
        }

        /// <summary>
        /// Gets or sets the drug types.
        /// </summary>
        /// <value>The drug types.</value>
        public IList<DrugType> DrugTypes
        {
            get
            {
                return this.drugTypes;
            }
            set
            {
                this.drugTypes = value;
                this.RefreshDrugType(value);
            }
        }

        /// <summary>
        /// Gets or sets the insurances.
        /// </summary>
        /// <value>The insurances.</value>
        public IList<Insurance> Insurances
        {
            get
            {
                return this.insurances;
            }
            set
            {
                this.insurances = value;
                this.RefreshInsurance(value);
            }
        }

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public object ItemToManage
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the pathologies.
        /// </summary>
        /// <value>The pathologies.</value>
        public IList<Pathology> Pathologies
        {
            get
            {
                return this.pathologies;
            }
            set
            {
                this.pathologies = value;
                this.RefreshPathology(value);
            }
        }

        /// <summary>
        /// Gets or sets the pathology types.
        /// </summary>
        /// <value>The pathology types.</value>
        public IList<PathologyType> PathologyTypes
        {
            get
            {
                return this.pathologyTypes;
            }
            set
            {
                this.pathologyTypes = value;
                this.RefreshPathologyType(value);
            }
        }

        /// <summary>
        /// Gets or sets the practices.
        /// </summary>
        /// <value>The practices.</value>
        public IList<Practice> Practices
        {
            get
            {
                return this.practices;
            }
            set
            {
                this.practices = value;
                this.RefreshPractice(value);
            }
        }

        /// <summary>
        /// Gets or sets the relation types.
        /// </summary>
        /// <value>The relation types.</value>
        public IList<RelationType> RelationTypes
        {
            get
            {
                return this.relationTypes;
            }
            set
            {
                this.relationTypes = value;
                this.RefreshRelationType(value);
            }
        }

        /// <summary>
        /// Gets or sets the reputations.
        /// </summary>
        /// <value>The reputations.</value>
        public IList<Reputation> Reputations
        {
            get { return this.reputations; }
            set
            {
                this.reputations = value;
                this.RefreshReputation(value);
            }
        }

        #endregion Properties

        #region Methods

        private void lv_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (sender == this.lv_Reputation)
            {
                if (this.lv_Reputation.SelectedItems.Count != 0)
                    this.OpenReputationForm(Usage.Update);
            }
            if (sender == this.lv_Doctor)
            {
                if (this.lv_Doctor.SelectedItems.Count != 0)
                    this.OpenDoctorForm(Usage.Update);
            }
            if (sender == this.lv_DoctorType)
            {
                if (this.lv_DoctorType.SelectedItems.Count != 0)
                    this.OpenDoctorTypeForm(Usage.Update);
            }
            if (sender == this.lv_Drugs)
            {
                if (this.lv_Drugs.SelectedItems.Count != 0)
                    this.OpenDrugForm(Usage.Update);
            }
            if (sender == this.lv_DrugType)
            {
                if (this.lv_DrugType.SelectedItems.Count != 0)
                    this.OpenDrugTypeForm(Usage.Update);
            }
            if (sender == this.lv_Insurance)
            {
                if (this.lv_Insurance.SelectedItems.Count != 0)
                    this.OpenInsuranceForm(Usage.Update);
            }
            if (sender == this.lv_Pathology)
            {
                if (this.lv_Pathology.SelectedItems.Count != 0)
                    this.OpenPathologyForm(Usage.Update);
            }
            if (sender == this.lv_PathologyType)
            {
                if (this.lv_PathologyType.SelectedItems.Count != 0)
                    this.OpenPathologyTypeForm(Usage.Update);
            }
            if (sender == this.lv_Practice)
            {
                if (this.lv_Practice.SelectedItems.Count != 0)
                    this.OpenPracticeForm(Usage.Update);
            }
            if (sender == this.lv_RelationType)
            {
                if (this.lv_RelationType.SelectedItems.Count != 0)
                    this.OpenRelationTypeForm(Usage.Update);
            }
        }

        private void OpenDoctorForm(Usage usage)
        {
            var box = GuiManager.Get<IDoctorInputPresenter>();
            box.Usage = usage;

            if (this.lv_Doctor.SelectedItems.Count > 0 && usage != Usage.Insert)
                box.ItemToManage = this.lv_Doctor.SelectedItems[0].Tag as Doctor;
            else if (usage != Usage.Insert)
            {
                MessageBoxes.Warning("Please select an item");
                return;
            }

            var dr = GuiManager.ShowDialog(box);
            if (dr != DialogResult.OK) return;
            this.Refreshed.Raise(this, new RefreshEventArgs(typeof(Doctor)));
        }

        private void OpenDoctorTypeForm(Usage usage)
        {
            var box = GuiManager.Get<IDoctorTypeInputPresenter>();
            box.Usage = usage;

            if (this.lv_DoctorType.SelectedItems.Count > 0 && usage != Usage.Insert)
                box.ItemToManage = this.lv_DoctorType.SelectedItems[0].Tag as DoctorType;
            else if (usage != Usage.Insert)
            {
                MessageBoxes.Warning("Please select an item");
                return;
            }

            var dr = GuiManager.ShowDialog(box);
            if (dr != DialogResult.OK) return;
            this.Refreshed.Raise(this, new RefreshEventArgs(typeof(DoctorType)));
        }

        private void OpenDrugForm(Usage usage)
        {
            var box = GuiManager.Get<IDrugInputPresenter>();
            box.Usage = usage;

            if (this.lv_Drugs.SelectedItems.Count > 0 && usage != Usage.Insert)
                box.ItemToManage = this.lv_Drugs.SelectedItems[0].Tag as Drug;
            else if (usage != Usage.Insert)
            {
                MessageBoxes.Warning("Please select an item");
                return;
            }

            var dr = GuiManager.ShowDialog(box);
            if (dr != DialogResult.OK) return;
            this.Refreshed.Raise(this, new RefreshEventArgs(typeof(Drug)));
        }

        private void OpenDrugTypeForm(Usage usage)
        {
            var box = GuiManager.Get<IDrugTypeInputPresenter>();
            box.Usage = usage;

            if (this.lv_DrugType.SelectedItems.Count > 0 && usage != Usage.Insert)
                box.ItemToManage = this.lv_DrugType.SelectedItems[0].Tag as DrugType;
            else if (usage != Usage.Insert)
            {
                MessageBoxes.Warning("Please select an item");
                return;
            }

            var dr = GuiManager.ShowDialog(box);
            if (dr != DialogResult.OK) return;
            this.Refreshed.Raise(this, new RefreshEventArgs(typeof(DrugType)));
        }

        private void OpenInsuranceForm(Usage usage)
        {
            var box = GuiManager.Get<IInsuranceInputPresenter>();
            box.Usage = usage;

            if (this.lv_Insurance.SelectedItems.Count > 0 && usage != Usage.Insert)
                box.ItemToManage = this.lv_Insurance.SelectedItems[0].Tag as Insurance;
            else if (usage != Usage.Insert)
            {
                MessageBoxes.Warning("Please select an item");
                return;
            }

            var dr = GuiManager.ShowDialog(box);
            if (dr != DialogResult.OK) return;
            this.Refreshed.Raise(this, new RefreshEventArgs(typeof(Insurance)));
        }

        private void OpenPathologyForm(Usage usage)
        {
            IPathologyInputPresenter box = GuiManager.Get<IPathologyInputPresenter>();
            box.Usage = usage;

            if (this.lv_Pathology.SelectedItems.Count > 0 && usage != Usage.Insert)
                box.ItemToManage = this.lv_Pathology.SelectedItems[0].Tag as Pathology;
            else if (usage != Usage.Insert)
            {
                MessageBoxes.Warning("Please select an item");
                return;
            }

            var dr = GuiManager.ShowDialog(box);
            if (dr != DialogResult.OK) return;
            this.Refreshed.Raise(this, new RefreshEventArgs(typeof(Pathology)));
        }

        private void OpenPathologyTypeForm(Usage usage)
        {
            var box = GuiManager.Get<IPathologyTypeInputPresenter>();
            box.Usage = usage;

            if (this.lv_PathologyType.SelectedItems.Count > 0 && usage != Usage.Insert)
                box.ItemToManage = this.lv_PathologyType.SelectedItems[0].Tag as PathologyType;
            else if (usage != Usage.Insert)
            {
                MessageBoxes.Warning("Please select an item");
                return;
            }

            var dr = GuiManager.ShowDialog(box);
            if (dr != DialogResult.OK) return;
            this.Refreshed.Raise(this, new RefreshEventArgs(typeof(PathologyType)));
        }

        private void OpenPracticeForm(Usage usage)
        {
            var box = GuiManager.Get<IPracticeInputPresenter>();
            box.Usage = usage;

            if (this.lv_Practice.SelectedItems.Count > 0 && usage != Usage.Insert)
                box.ItemToManage = this.lv_Practice.SelectedItems[0].Tag as Practice;
            else if (usage != Usage.Insert)
            {
                MessageBoxes.Warning("Please select an item");
                return;
            }

            var dr = GuiManager.ShowDialog(box);
            if (dr != DialogResult.OK) return;
            this.Refreshed.Raise(this, new RefreshEventArgs(typeof(Practice)));
        }

        private void OpenRelationTypeForm(Usage usage)
        {
            var box = GuiManager.Get<IRelationTypeInputPresenter>();
            box.Usage = usage;

            if (this.lv_RelationType.SelectedItems.Count > 0 && usage != Usage.Insert)
                box.ItemToManage = this.lv_RelationType.SelectedItems[0].Tag as RelationType;
            else if (usage != Usage.Insert)
            {
                MessageBoxes.Warning("Please select an item");
                return;
            }

            var dr = GuiManager.ShowDialog(box);
            if (dr != DialogResult.OK) return;
            this.Refreshed.Raise(this, new RefreshEventArgs(typeof(RelationType)));
        }

        private void OpenReputationForm(Usage usage)
        {
            var box = GuiManager.Get<IReputationInputPresenter>();
            box.Usage = usage;

            if (this.lv_Reputation.SelectedItems.Count > 0 && usage != Usage.Insert)
                box.ItemToManage = this.lv_Reputation.SelectedItems[0].Tag as Reputation;
            else if (usage != Usage.Insert)
            {
                MessageBoxes.Warning("Please select an item");
                return;
            }

            var dr = GuiManager.ShowDialog(box);
            if (dr != DialogResult.OK) return;
            this.Refreshed.Raise(this, new RefreshEventArgs(typeof(Reputation)));
        }

        private void ProcessClick(Usage usage)
        {
            if (this.tab_Reputation.Visible) this.OpenReputationForm(usage);
            else if (this.tab_Insurance.Visible) this.OpenInsuranceForm(usage);
            else if (this.tab_Practice.Visible) this.OpenPracticeForm(usage);
            else if (this.tab_Doctor.Visible) this.OpenDoctorForm(usage);
            else if (this.tab_DoctorType.Visible) this.OpenDoctorTypeForm(usage);
            else if (this.tab_RelationType.Visible) this.OpenRelationTypeForm(usage);
            else if (this.tab_Drug.Visible) this.OpenDrugForm(usage);
            else if (this.tab_DrugType.Visible) this.OpenDrugTypeForm(usage);
            else if (this.tab_Pathology.Visible) this.OpenPathologyForm(usage);
            else if (this.tab_PathologyType.Visible) this.OpenPathologyTypeForm(usage);
        }

        private void RefreshDoctor(IList<Doctor> list)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { RefreshDoctor(list); });
            }
            else
            {
                this.lv_Doctor.Items.Clear();
                foreach (var item in list)
                {
                    ListViewItem listViewItem = new ListViewItem(item.LastName + " " + item.FirstName);
                    listViewItem.Tag = item;
                    listViewItem.ImageKey = "Doctor";
                    this.lv_Doctor.Items.Add(listViewItem);
                }
            }
        }

        private void RefreshDoctorType(IList<DoctorType> list)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { RefreshDoctorType(list); });
            }
            else
            {
                this.lv_DoctorType.Items.Clear();
                foreach (var item in list)
                {
                    ListViewItem listViewItem = new ListViewItem(item.Title);
                    listViewItem.Tag = item;
                    listViewItem.ImageKey = "DoctorType";
                    this.lv_DoctorType.Items.Add(listViewItem);
                }
            }
        }

        private void RefreshDrug(IList<Drug> list)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { RefreshDrug(list); });
            }
            else
            {
                this.lv_Drugs.Items.Clear();
                foreach (var item in list)
                {
                    ListViewItem listViewItem = new ListViewItem(item.Title);
                    listViewItem.Tag = item;
                    listViewItem.ImageKey = "Drug";
                    this.lv_Drugs.Items.Add(listViewItem);
                }
            }
        }

        private void RefreshDrugType(IList<DrugType> list)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { RefreshDrugType(list); });
            }
            else
            {
                this.lv_DrugType.Items.Clear();
                foreach (var item in list)
                {
                    ListViewItem listViewItem = new ListViewItem(item.Title);
                    listViewItem.Tag = item;
                    listViewItem.ImageKey = "DrugType";
                    this.lv_DrugType.Items.Add(listViewItem);
                }
            }
        }

        private void RefreshInsurance(IList<Insurance> list)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { RefreshInsurance(list); });
            }
            else
            {
                this.lv_Insurance.Items.Clear();
                foreach (var item in list)
                {
                    ListViewItem listViewItem = new ListViewItem(item.Title);
                    listViewItem.Tag = item;
                    listViewItem.ImageKey = "Insurance";
                    this.lv_Insurance.Items.Add(listViewItem);
                }
            }
        }

        private void RefreshPathology(IList<Pathology> list)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { RefreshPathology(list); });
            }
            else
            {
                this.lv_Pathology.Items.Clear();
                foreach (var item in list)
                {
                    ListViewItem listViewItem = new ListViewItem(item.Title);
                    listViewItem.Tag = item;
                    listViewItem.ImageKey = "Pathology";
                    this.lv_Pathology.Items.Add(listViewItem);
                }
            }
        }

        private void RefreshPathologyType(IList<PathologyType> list)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { RefreshPathologyType(list); });
            }
            else
            {
                this.lv_PathologyType.Items.Clear();
                foreach (var item in list)
                {
                    ListViewItem listViewItem = new ListViewItem(item.Title);
                    listViewItem.Tag = item;
                    listViewItem.ImageKey = "PathologyType";
                    this.lv_PathologyType.Items.Add(listViewItem);
                }
            }
        }

        private void RefreshPractice(IList<Practice> list)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { RefreshPractice(list); });
            }
            else
            {
                this.lv_Practice.Items.Clear();
                foreach (var item in list)
                {
                    ListViewItem listViewItem = new ListViewItem(item.Title);
                    listViewItem.Tag = item;
                    listViewItem.ImageKey = "Practice";
                    this.lv_Practice.Items.Add(listViewItem);
                }
            }
        }

        private void RefreshRelationType(IList<RelationType> list)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { RefreshRelationType(list); });
            }
            else
            {
                this.lv_RelationType.Items.Clear();
                foreach (var item in list)
                {
                    ListViewItem listViewItem = new ListViewItem(item.Title);
                    listViewItem.Tag = item;
                    listViewItem.ImageKey = "RelationType";
                    this.lv_RelationType.Items.Add(listViewItem);
                }
            }
        }

        private void RefreshReputation(IList<Reputation> list)
        {
            if (this.InvokeRequired)
            {
                this.Invoke((Action)delegate { RefreshReputation(list); });
            }
            else
            {
                this.lv_Reputation.Items.Clear();
                foreach (var reputation in list)
                {
                    ListViewItem item = new ListViewItem(reputation.Title);
                    item.Tag = reputation;
                    item.ImageKey = "Reputation";
                    this.lv_Reputation.Items.Add(item);
                }
            }
        }

        private void tab_Enter(object sender, EventArgs e)
        {
            Type type = typeof(object);

            if (sender == this.tab_Reputation) type = typeof(Reputation);
            else if (sender == this.tab_Insurance) type = typeof(Insurance);
            else if (sender == this.tab_Practice) type = typeof(Practice);
            else if (sender == this.tab_Doctor) type = typeof(Doctor);
            else if (sender == this.tab_DoctorType) type = typeof(DoctorType);
            else if (sender == this.tab_RelationType) type = typeof(RelationType);
            else if (sender == this.tab_Drug) type = typeof(Drug);
            else if (sender == this.tab_DrugType) type = typeof(DrugType);
            else if (sender == this.tab_Pathology) type = typeof(Pathology);
            else if (sender == this.tab_PathologyType) type = typeof(PathologyType);
            else Assert.Fails("The tab '{0}' is not supported", sender);

            var args = new RefreshEventArgs(type);
            this.Refreshed.Raise(this, args);
        }

        private void tsb_Add_Click(object sender, EventArgs e)
        {
            ProcessClick(Usage.Insert);
        }

        private void tsb_Delete_Click(object sender, EventArgs e)
        {
            this.ProcessClick(Usage.Delete);
        }

        private void tsb_Update_Click(object sender, EventArgs e)
        {
            this.ProcessClick(Usage.Update);
        }

        #endregion Methods
    }
}