﻿namespace Phoenix.View.Frame.Configuration
{
    using System.Windows.Forms;

    using Phoenix.Translation;
    using Phoenix.View.Configuration;

    /// <summary>
    /// Provides generic appearance configuration.
    /// </summary>
    public partial class AppearanceConfigurationControl : UserControl, IConfigurationControl
    {
        #region Fields

        private AppearanceConfiguration appearanceConfiguration = new AppearanceConfiguration();
        private string TipMinimumDate = "This date is the minimum date Phoenix can handle.".Translate();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="AppearanceConfigurationControl"/> class.
        /// </summary>
        public AppearanceConfigurationControl()
        {
            InitializeComponent();
            this.lbl_MinDate.Text = "Minimum date:".Translate();
            this.tt_Tips.SetToolTip(this.lbl_MinDate, TipMinimumDate.Translate());
            this.tt_Tips.SetToolTip(this.dtp_MinDate, TipMinimumDate.Translate());
            this.dtp_MinDate.Value = this.appearanceConfiguration.MinDate;
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Rollback theconfiguration date to the previous state.
        /// </summary>
        public void Rollback()
        {
            this.appearanceConfiguration.Rollback();
        }

        /// <summary>
        /// Saves the configuration in the config file.
        /// </summary>
        public void SaveConfiguration()
        {
            this.appearanceConfiguration.MinDate = this.dtp_MinDate.Value;
            this.appearanceConfiguration.CommitChanges();
        }

        #endregion Methods
    }
}