﻿namespace Phoenix.View.Helpers.Controls
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;
    using System.Data;
    using System.Drawing;
    using System.Linq;
    using System.Text;
    using System.Windows.Forms;

    /// <summary>
    /// NumericUpDown with title
    /// </summary>
    public partial class TitledNumericUpDown : UserControl, ITitledControl
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TitledNumericUpDown"/> class.
        /// </summary>
        public TitledNumericUpDown()
        {
            InitializeComponent();
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets the data bindings for the control.
        /// </summary>
        /// <value></value>
        /// <returns>
        /// A <see cref="T:System.Windows.Forms.ControlBindingsCollection"/> that contains the <see cref="T:System.Windows.Forms.Binding"/> objects for the control.
        /// </returns>
        public new ControlBindingsCollection DataBindings
        {
            get { return this.numericUpDown.DataBindings; }
        }

        /// <summary>
        /// Gets or sets the width of the label's column.
        /// </summary>
        /// <value>The width of the label.</value>
        public float LabelWidth
        {
            get
            {
                return this.tlp_General.ColumnStyles[0].Width;
            }
            set
            {
                this.tlp_General.ColumnStyles[0].Width = value;
            }
        }

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        /// <value>The title.</value>
        public string Title
        {
            get
            {
                return this.label.Text;
            }
            set
            {
                this.label.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        /// <value>The value.</value>
        public decimal Value
        {
            get
            {
                return this.numericUpDown.Value;
            }
            set
            {
                this.numericUpDown.Value = value;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Adds a new data binding.
        /// </summary>
        /// <param name="dataSource">The data source.</param>
        /// <param name="dataMember">The data member.</param>
        public void AddDataBinding(object dataSource, string dataMember)
        {
            this.numericUpDown.DataBindings.Add("Text", dataSource, dataMember);
        }

        /// <summary>
        /// Clears the data binding.
        /// </summary>
        public void ClearDataBinding()
        {
            this.numericUpDown.DataBindings.Clear();
        }

        #endregion Methods
    }
}