﻿namespace Phoenix.View.Helpers.Controls
{
    using System.Windows.Forms;

    /// <summary>
    /// Represent a text box with a title.
    /// </summary>
    /// <remarks>This class is sealed because it overrides the property DataBindings whose is not virtual. </remarks>
    public sealed partial class TitledTextBox : UserControl, ITitledControl
    {
        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TitledTextBox"/> class.
        /// </summary>
        public TitledTextBox()
        {
            InitializeComponent();
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets the data bindings for the control.
        /// </summary>
        /// <value></value>
        /// <returns>
        /// A <see cref="T:System.Windows.Forms.ControlBindingsCollection"/> that contains the <see cref="T:System.Windows.Forms.Binding"/> objects for the control.
        /// </returns>
        public new ControlBindingsCollection DataBindings
        {
            get { return this.textBox.DataBindings; }
        }

        /// <summary>
        /// Gets or sets the width of the label's column.
        /// </summary>
        /// <value>The width of the label.</value>
        public float LabelWidth
        {
            get
            {
                return this.tlp_General.ColumnStyles[0].Width;
            }
            set
            {
                this.tlp_General.ColumnStyles[0].Width = value;
            }
        }

        /// <summary>
        /// Gets or sets the title.
        /// </summary>
        /// <value>The title.</value>
        public string Title
        {
            get
            {
                return this.label.Text;
            }
            set
            {
                this.label.Text = value;
            }
        }

        /// <summary>
        /// Gets or sets the value.
        /// </summary>
        /// <value>The value.</value>
        public string Value
        {
            get
            {
                return this.textBox.Text;
            }
            set
            {
                this.textBox.Text = value;
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Adds a new data binding.
        /// </summary>
        /// <param name="dataSource">The data source.</param>
        /// <param name="dataMember">The data member.</param>
        public void AddDataBinding(object dataSource, string dataMember)
        {
            this.textBox.DataBindings.Add("Text", dataSource, dataMember);
        }

        /// <summary>
        /// Clears the data binding.
        /// </summary>
        public void ClearDataBinding()
        {
            this.textBox.DataBindings.Clear();
        }

        #endregion Methods
    }
}