﻿namespace Phoenix.View.Input
{
    using Phoenix.Contracts;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Interfaces.Input;
    using Phoenix.Translation;
    using Phoenix.View.Helpers;

    /// <summary>
    /// Manage the drug types
    /// </summary>
    public class DrugTypeInput : BasicInput, IDrugTypeInputPresenter
    {
        #region Fields

        private const string VALUE1 = "Title";
        private const string VALUE2 = "Remark";

        private DrugTypeModel presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="DrugTypeInput"/> class.
        /// </summary>
        public DrugTypeInput()
            : base(new DrugType(), VALUE1, VALUE2)
        {
            this.presenter = new DrugTypeModel(this);
            this.Title = "Manage drug types".Translate();
        }

        #endregion Constructors

        #region Properties

        /// <summary>
        /// Gets or sets the item to manage.
        /// </summary>
        /// <value>The item to manage.</value>
        public DrugType ItemToManage
        {
            get
            {
                return this.Item as DrugType;
            }
            set
            {
                var temp = value as DrugType;
                this.Bind(value, VALUE1, VALUE2);

            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Sets an error message on validation failed.
        /// </summary>
        /// <param name="fail">Indicates where the validation failed.</param>
        /// <param name="message">The message to display.</param>
        public void SetError(DrugTypeFailedOn fail, string message)
        {
            switch (fail)
            {
                case DrugTypeFailedOn.NoName:
                    this.SetError(On.Value1, message);
                    break;
                case DrugTypeFailedOn.Referenced:
                    PredifinedMsgBoxes.WarnOnReferencedItem();
                    break;
                default:
                    Assert.FailsOnEnumeration(fail);
                    break;
            }
        }

        /// <summary>
        /// Validates the data.
        /// </summary>
        /// <returns>
        /// Returns <c>true</c> if the data is valid; otherwise <b>false</b>
        /// </returns>
        protected override bool ValidateData()
        {
            return this.presenter.ValidateData();
        }

        #endregion Methods
    }
}