﻿namespace Phoenix.Presenter.Management
{
    using System;
    using System.Windows.Forms;

    using Phoenix.Helpers.Misc;
    using Phoenix.Helpers.Winforms;
    using Phoenix.Model.Bll.Domain;
    using Phoenix.Model.Management;
    using Phoenix.Presenter.Interfaces;
    using Phoenix.Translation;
    using Phoenix.View.Helpers;

    /// <summary>
    /// This box provides patient deletion feature.
    /// </summary>
    public class PatientDeletionBox : IDeleter<Patient>
    {
        #region Fields

        private PatientDeleterModel presenter;

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="PatientDeletionBox"/> class.
        /// </summary>
        /// <param name="patient">The patient.</param>
        public PatientDeletionBox(Patient patient)
        {
            this.ItemToDelete = patient;
            this.presenter = new PatientDeleterModel(this);
        }

        #endregion Constructors

        #region Events

        /// <summary>
        /// Occurs when the user asked a deletion of an item.
        /// </summary>
        public event EventHandler<EventArgs<Patient>> Deleted;

        #endregion Events

        #region Properties

        private Patient ItemToDelete
        {
            get;
            set;
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Shows the form as a modal dialog box with the currently active window set
        /// as its owner.
        /// </summary>
        public DialogResult ShowDialog()
        {
            CaptchaBox fr = new CaptchaBox();
            DialogResult dr = DialogResult.Cancel;
            fr.Explanation = "To confirm you really want to delete this patient and all data linked"
                + " to him/her please enter the code displayed below in the textbox."
                + " The patient will be deleted and it will be impossible to rollback!".Translate();
            fr.ConfirmationSucceeded += (sender, e) =>
            {
                this.Deleted.Raise(this, new EventArgs<Patient>(this.ItemToDelete));
                dr = DialogResult.OK;
            };
            fr.ConfirmationFailed += (sender, e) =>
            {
                MessageBoxes.Exclamation("The confirmation code is not correct. Please try again!".Translate());

            };
            fr.ShowDialog();
            return dr;
        }

        #endregion Methods
    }
}