﻿namespace Phoenix.Helpers.Winforms.Logging
{
    using System;
    using System.Diagnostics;
    using System.Windows.Forms;

    /// <summary>
    /// Provide a listenenr that logs into a textbox
    /// </summary>
    public class TextBoxTraceListener : TraceListener
    {
        #region Fields

        private TextBox textBox; // Text box that displays the trace messages.

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes a new instance of the <see cref="TextBoxTraceListener"/> class.
        /// </summary>
        /// <param name="textBox">The text box.</param>
        public TextBoxTraceListener(TextBox textBox)
        {
            this.textBox = textBox;
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// You define these and put the message into your textbox (traceEdit).
        /// </summary>
        /// <param name="message"></param>
        public override void Write(String message)
        {
            if (!this.textBox.IsDisposed)
                this.AppendText(message);
        }

        /// <summary>
        /// You define these and put the message into your textbox (traceEdit).
        /// </summary>
        /// <param name="message">A message to write.</param>
        public override void WriteLine(String message)
        {
            if (!this.textBox.IsDisposed)
                this.AppendText(message);
        }

        private void AppendText(string message)
        {
            if (this.textBox.InvokeRequired)
            {
                this.textBox.Invoke((Action)delegate { this.AppendText(message); });
            }
            else
            {
                this.textBox.AppendText(message);
            }
        }

        #endregion Methods
    }
}