﻿namespace Phoenix.Helpers.Misc
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel;

    /// <summary>
    /// Get the description of the specified enumeration
    /// </summary>
    public static class EnumHelper
    {
        #region Methods

        /// <summary>
        /// Counts the number of items the specified Enum has got.
        /// </summary>
        /// <param name="enumType">The value.</param>
        /// <returns>The number of value has got the Enum</returns>
        public static int Count(Type enumType)
        {
            return Enum.GetValues(enumType).Length;
        }

        /// <summary>
        /// Gets the descriptions of the current Enum
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns></returns>
        public static string Description(this Enum value)
        {
            return GetDescription(value);
        }

        /// <summary>
        /// Gets the description of the specified Enum.
        /// </summary>
        /// <param name="value">The Enum whereon extract the description.</param>
        /// <returns>A text representing the description of the Enum</returns>
        public static string GetDescription(Enum value)
        {
            // Check for Enum that is marked with FlagAttribute
            var entries = value.ToString().Split(' ');
            var description = new string[entries.Length];

            for (var i = 0; i < entries.Length; i++)
            {
                var fieldInfo = value.GetType().GetField(entries[i].Trim().Trim(','));

                if (fieldInfo == null)
                {
                    return value.ToString();
                }
                else
                {
                    var attributes = (DescriptionAttribute[])fieldInfo.GetCustomAttributes(typeof(DescriptionAttribute), false);
                    description[i] = (attributes.Length > 0) ? attributes[0].Description : entries[i].Trim();
                }
            }
            return String.Join(", ", description);
        }

        /// <summary>
        /// Gets the list of descriptions contained in the specified Enum.
        /// </summary>
        /// <param name="enumType">The value.</param>
        /// <returns></returns>
        public static Dictionary<Enum, string> GetDescriptions(Type enumType)
        {
            Dictionary<Enum, string> result = new Dictionary<Enum, string>();
            foreach (var item in Enum.GetValues(enumType))
            {
                Enum e = item as Enum;
                if (e == null)
                {
                    throw new NotSupportedException(string.Format(
                        "The specified type should be derived 'Enum' but it is '{0}'", enumType.Name));
                }
                result.Add(e, e.Description());
            }
            return result;
        }

        #endregion Methods
    }
}