﻿namespace Phoenix.S2P.Helpers
{
    using System.Configuration;
    using System.Xml;

    using Phoenix.Model.Bll.Domain;

    public class DbInsert
    {
        #region Properties

        public DoctorType DefaultSpecialisation
        {
            get; set;
        }

        public DoctorType Doctor
        {
            get; set;
        }

        public DoctorType Homeopath
        {
            get; set;
        }

        public Practice Practice
        {
            get;
            set;
        }

        public User User
        {
            get;
            set;
        }

        #endregion Properties
    }

    public class DBInsertSectionHandler : IConfigurationSectionHandler
    {
        #region Fields

        int addressCount = 0;
        int doctorCount = 0;
        int doctorTypeCount = 0;
        int homeopathCount = 0;
        int mainPracticeCount = 0;
        int userCount = 0;

        #endregion Fields

        #region Methods

        public object Create(object parent, object configContext, XmlNode section)
        {
            DbInsert result = new DbInsert();

            foreach (XmlNode node in section.ChildNodes)
            {
                if (node.Name == "MainPractice")
                    result.Practice = GetMainPractice(node);
                if (node.Name == "User")
                    result.User = GetUser(node, result.Practice);
                if (node.Name == "DefaultSpeciality")
                    result.DefaultSpecialisation = GetDoctorType(node);
                if (node.Name == "Homeopath")
                    result.Homeopath = GetHomeopath(node);
                if (node.Name == "Doctor")
                    result.Doctor = GetDoctor(node);
            }
            return result;
        }

        private DoctorType GetDoctor(XmlNode node)
        {
            if (this.doctorCount++ > 0)
                throw new ConfigurationErrorsException("Only one node named 'Doctor' is allowed!");
            if (node.Attributes["name"] != null)
                return new DoctorType() { Title = node.Attributes["name"].Value };
            else
                return new DoctorType() { Title = "Doctor" };
        }

        private DoctorType GetDoctorType(XmlNode node)
        {
            if (this.doctorTypeCount++ > 0)
                throw new ConfigurationErrorsException("Only one node named 'DefaultSpeciality' is allowed!");
            if (node.Attributes["name"] != null)
                return new DoctorType() { Title = node.Attributes["name"].Value };
            else
                return new DoctorType() { Title = "Without speciality" };
        }

        private DoctorType GetHomeopath(XmlNode node)
        {
            if (this.homeopathCount++ > 0)
                throw new ConfigurationErrorsException("Only one node named 'Homeopath' is allowed!");
            if (node.Attributes["name"] != null)
                return new DoctorType() { Title = node.Attributes["name"].Value };
            else
                return new DoctorType() { Title = "Homeopath" };
        }

        private Practice GetMainPractice(XmlNode node)
        {
            string title = string.Empty;
            string phone = string.Empty;

            string city = string.Empty;
            string postalCode = string.Empty;
            string street = string.Empty;
            string streetNumber = string.Empty;
            string boxNumber = string.Empty;
            Address address = null;
            Practice practice = new Practice();

            if (this.mainPracticeCount++ > 0)
                throw new ConfigurationErrorsException("Only one node named 'MainPractice' is allowed!");

            practice = new Practice();

            if (node.Attributes["Title"] != null)
                title = node.Attributes["Title"].Value;

            if (node.Attributes["Phone"] != null)
                phone = node.Attributes["Phone"].Value;

            foreach (XmlNode innerNode in node.ChildNodes)
            {
                if (innerNode.Name != "Address") continue;

                address = new Address();

                if (addressCount++ > 0)
                    throw new ConfigurationErrorsException("Only one node named 'Address' is allowed!");

                if (innerNode.Attributes["City"] != null)
                    city = innerNode.Attributes["City"].Value;

                if (innerNode.Attributes["PostalCode"] != null)
                    postalCode = innerNode.Attributes["PostalCode"].Value;

                if (innerNode.Attributes["Street"] != null)
                    street = innerNode.Attributes["Street"].Value;

                if (innerNode.Attributes["StreetNumber"] != null)
                    streetNumber = innerNode.Attributes["StreetNumber"].Value;

                if (innerNode.Attributes["BoxNumber"] != null)
                    boxNumber = innerNode.Attributes["BoxNumber"].Value;

                address.City = city;
                address.PostalCode = postalCode;
                address.Street = street;
                address.StreetNumber = streetNumber;
                address.BoxNumber = boxNumber;

            }

            practice.Address = address;
            practice.Phone = phone;
            practice.Title = title;

            return (this.mainPracticeCount == 0)
                ? null
                : practice;
        }

        private User GetUser(XmlNode node, Practice practice)
        {
            decimal fee = 0;
            string firstName = string.Empty;
            string lastName = string.Empty;
            string header = string.Empty;
            User user = new User();

            if (this.userCount++ > 0)
                throw new ConfigurationErrorsException("Only one node named 'User' is allowed!");

            if (node.Attributes["Fee"] != null)
                fee = decimal.Parse(node.Attributes["Fee"].Value);

            if (node.Attributes["FirstName"] != null)
                firstName = node.Attributes["FirstName"].Value;

            if (node.Attributes["LastName"] != null)
                lastName = node.Attributes["LastName"].Value;

            if (node.Attributes["Header"] != null)
                header = node.Attributes["Header"].Value;

            user.FirstName = firstName;
            user.LastName = lastName;
            user.Fee = fee;
            user.Header = header;
            user.MainPractice = practice;

            return (this.userCount == 0)
                ? null
                : user;
        }

        #endregion Methods
    }
}